@@ -1,7 +1,7 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2004, 2005, 2006,
-   2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2004, 2005, 2006, 2007
+   Free Software Foundation, Inc.
 
    This file is part of GNU M4.
 
@@ -22,12 +22,12 @@
 #include "m4.h"
 
 #include <getopt.h>
+#include <limits.h>
 #include <signal.h>
-#include <stdarg.h>
 
 #include "version-etc.h"
 
-#define AUTHORS "Rene' Seindal", "Eric Blake"
+#define AUTHORS "Rene' Seindal"
 
 static void usage (int);
 
@@ -47,7 +47,7 @@ int no_gnu_extensions = 0;
 int prefix_all_builtins = 0;
 
 /* Max length of arguments in trace output (-lsize).  */
-size_t max_debug_argument_length = SIZE_MAX;
+int max_debug_argument_length = 0;
 
 /* Suppress warnings about missing arguments.  */
 int suppress_warnings = 0;
@@ -83,135 +83,34 @@ typedef struct macro_definition macro_definition;
 
 /* Error handling functions.  */
 
-/*------------------------------------------------------------------.
-| Helper for all the error reporting, as a wrapper around	    |
-| error_at_line.  Report error message based on FORMAT and ARGS, on |
-| behalf of MACRO, at the location FILE and LINE (but with no	    |
-| location if LINE is 0).  If ERRNUM, decode the errno value that   |
-| caused the error.  If STATUS, exit immediately with that status.  |
-| If WARN, prepend 'Warning: '.					    |
-`------------------------------------------------------------------*/
+/*-----------------------.
+| Wrapper around error.  |
+`-----------------------*/
 
-static void
-m4_verror_at_line (bool warn, int status, int errnum, const char *file,
-		   int line, const char *macro, const char *format,
-		   va_list args)
+void
+m4_error (int status, int errnum, const char *format, ...)
 {
-  char *full = NULL;
-  char *safe_macro = NULL;
-
-  /* Sanitize MACRO, since we are turning around and using it in a
-     format string.  The allocation is overly conservative, but
-     problematic macro names only occur via indir or changeword.  */
-  if (macro && strchr (macro, '%'))
-    {
-      char *p = safe_macro = xcharalloc (2 * strlen (macro) + 1);
-      do
-	{
-	  if (*macro == '%')
-	    *p++ = '%';
-	  *p++ = *macro++;
-	}
-      while (*macro);
-      *p = '\0';
-    }
-  /* Prepend warning and the macro name, as needed.  But if that fails
-     for non-memory reasons (unlikely), then still use the original
-     format.  */
-  if (warn && macro)
-    full = xasprintf (_("Warning: %s: %s"),
-		      quotearg (safe_macro ? safe_macro : macro), format);
-  else if (warn)
-    full = xasprintf (_("Warning: %s"), format);
-  else if (macro)
-    full = xasprintf (_("%s: %s"),
-		      quotearg (safe_macro ? safe_macro : macro), format);
-  verror_at_line (status, errnum, line ? file : NULL, line,
-		  full ? full : format, args);
-  free (full);
-  free (safe_macro);
-  if ((!warn || fatal_warnings) && !retcode)
+  va_list args;
+  va_start (args, format);
+  verror_at_line (status, errnum, current_line ? current_file : NULL,
+		  current_line, format, args);
+  if (fatal_warnings && ! retcode)
     retcode = EXIT_FAILURE;
 }
 
-/*----------------------------------------------------------------.
-| Wrapper around error.  Report error message based on FORMAT and |
-| subsequent args, on behalf of MACRO, and the current input line |
-| (if any).  If ERRNUM, decode the errno value that caused the	  |
-| error.  If STATUS, exit immediately with that status.		  |
-`----------------------------------------------------------------*/
-
-void
-m4_error (int status, int errnum, const char *macro, const char *format, ...)
-{
-  va_list args;
-  va_start (args, format);
-  if (status == EXIT_SUCCESS && warning_status)
-    status = EXIT_FAILURE;
-  m4_verror_at_line (false, status, errnum, current_file, current_line,
-		     macro, format, args);
-  va_end (args);
-}
-
-/*----------------------------------------------------------------.
-| Wrapper around error_at_line.  Report error message based on	  |
-| FORMAT and subsequent args, on behalf of MACRO, at the location |
-| FILE and LINE (but with no location if LINE is 0).  If ERRNUM,  |
-| decode the errno value that caused the error.  If STATUS, exit  |
-| immediately with that status.					  |
-`----------------------------------------------------------------*/
+/*-------------------------------.
+| Wrapper around error_at_line.  |
+`-------------------------------*/
 
 void
 m4_error_at_line (int status, int errnum, const char *file, int line,
-		  const char *macro, const char *format, ...)
+		  const char *format, ...)
 {
   va_list args;
   va_start (args, format);
-  if (status == EXIT_SUCCESS && warning_status)
-    status = EXIT_FAILURE;
-  m4_verror_at_line (false, status, errnum, file, line, macro, format, args);
-  va_end (args);
-}
-
-/*------------------------------------------------------------------.
-| Wrapper around error.  Report warning message based on FORMAT and |
-| subsequent args, on behalf of MACRO, and the current input line   |
-| (if any).  If ERRNUM, decode the errno value that caused the	    |
-| warning.							    |
-`------------------------------------------------------------------*/
-
-void
-m4_warn (int errnum, const char *macro, const char *format, ...)
-{
-  va_list args;
-  if (!suppress_warnings)
-    {
-      va_start (args, format);
-      m4_verror_at_line (true, warning_status, errnum, current_file,
-			 current_line, macro, format, args);
-      va_end (args);
-    }
-}
-
-/*----------------------------------------------------------------.
-| Wrapper around error_at_line.  Report warning message based on  |
-| FORMAT and subsequent args, on behalf of MACRO, at the location |
-| FILE and LINE (but with no location if LINE is 0).  If ERRNUM,  |
-| decode the errno value that caused the warning.		  |
-`----------------------------------------------------------------*/
-
-void
-m4_warn_at_line (int errnum, const char *file, int line, const char *macro,
-		 const char *format, ...)
-{
-  va_list args;
-  if (!suppress_warnings)
-    {
-      va_start (args, format);
-      m4_verror_at_line (true, warning_status, errnum, file, line, macro,
-			 format, args);
-      va_end (args);
-    }
+  verror_at_line (status, errnum, line ? file : NULL, line, format, args);
+  if (fatal_warnings && ! retcode)
+    retcode = EXIT_FAILURE;
 }
 
 #ifdef USE_STACKOVF
@@ -223,8 +122,8 @@ m4_warn_at_line (int errnum, const char *file, int line, const char *macro,
 static void
 stackovf_handler (void)
 {
-  m4_error (EXIT_FAILURE, 0, NULL,
-	    _("ERROR: stack overflow.  (Infinite define recursion?)"));
+  M4ERROR ((EXIT_FAILURE, 0,
+	    "ERROR: stack overflow.  (Infinite define recursion?)"));
 }
 
 #endif /* USE_STACKOV */
@@ -419,10 +318,6 @@ process_file (const char *name)
 #define OPTSTRING "-B:D:EF:GH:I:L:N:PQR:S:T:U:d::eil:o:st:"
 #endif
 
-#ifdef DEBUG_REGEX
-FILE *trace_file;
-#endif /* DEBUG_REGEX */
-
 int
 main (int argc, char *const *argv, char *const *envp)
 {
@@ -443,20 +338,8 @@ main (int argc, char *const *argv, char *const *envp)
   retcode = EXIT_SUCCESS;
   atexit (close_stdin);
 
-#ifdef DEBUG_REGEX
-  {
-    const char *name = getenv ("M4_TRACE_FILE");
-    if (name)
-      trace_file = fopen (name, "a");
-    if (trace_file)
-      fputs ("m4:\n", trace_file);
-  }
-#endif /* DEBUG_REGEX */
-
   include_init ();
   debug_init ();
-  set_quoting_style (NULL, escape_quoting_style);
-  set_char_quoting (NULL, ':', 1);
 #ifdef USE_STACKOVF
   setup_stackovf_trap (argv, envp, stackovf_handler);
 #endif
@@ -509,7 +392,7 @@ main (int argc, char *const *argv, char *const *envp)
 	break;
 
       case 'E':
-	if (!fatal_warnings)
+	if (! fatal_warnings)
 	  fatal_warnings = true;
 	else
 	  warning_status = EXIT_FAILURE;
@@ -572,10 +455,9 @@ main (int argc, char *const *argv, char *const *envp)
 	break;
 
       case 'l':
-	{
-	  int tmp = atoi (optarg);
-	  max_debug_argument_length = tmp <= 0 ? SIZE_MAX : (size_t) tmp;
-	}
+	max_debug_argument_length = atoi (optarg);
+	if (max_debug_argument_length <= 0)
+	  max_debug_argument_length = 0;
 	break;
 
       case 'o':
@@ -589,11 +471,11 @@ main (int argc, char *const *argv, char *const *envp)
 	break;
 
       case WARN_MACRO_SEQUENCE_OPTION:
-	 /* Don't call set_macro_sequence here, as it can exit.
-	    --warn-macro-sequence sets optarg to NULL (which uses the
-	    default regexp); --warn-macro-sequence= sets optarg to ""
-	    (which disables these warnings).  */
-	macro_sequence = optarg;
+         /* Don't call set_macro_sequence here, as it can exit.
+            --warn-macro-sequence sets optarg to NULL (which uses the
+            default regexp); --warn-macro-sequence= sets optarg to ""
+            (which disables these warnings).  */
+        macro_sequence = optarg;
 	break;
 
       case VERSION_OPTION:
@@ -609,8 +491,8 @@ main (int argc, char *const *argv, char *const *envp)
   defines = head;
 
   /* Do the basic initializations.  */
-  if (debugfile && !debug_set_output (NULL, debugfile))
-    m4_error (0, errno, NULL, _("cannot set debug file `%s'"), debugfile);
+  if (debugfile && !debug_set_output (debugfile))
+    M4ERROR ((0, errno, "cannot set debug file `%s'", debugfile));
 
   input_init ();
   output_init ();
@@ -648,8 +530,7 @@ main (int argc, char *const *argv, char *const *envp)
 	    char *macro_value = strchr (macro_name, '=');
 	    if (macro_value)
 	      *macro_value++ = '\0';
-	    define_user_macro (macro_name, strlen (macro_name),
-			       macro_value, SYMBOL_INSERT);
+	    define_user_macro (macro_name, macro_value, SYMBOL_INSERT);
 	    free (macro_name);
 	  }
 	  break;
@@ -669,11 +550,11 @@ main (int argc, char *const *argv, char *const *envp)
 
 	case '\1':
 	  seen_file = true;
-	  process_file (defines->arg);
+          process_file (defines->arg);
 	  break;
 
 	default:
-	  assert (!"main");
+	  M4ERROR ((0, 0, "INTERNAL ERROR: bad code in deferred arguments"));
 	  abort ();
 	}
 
@@ -699,7 +580,7 @@ main (int argc, char *const *argv, char *const *envp)
   /* Change debug stream back to stderr, to force flushing the debug
      stream and detect any errors it might have encountered.  The
      three standard streams are closed by close_stdin.  */
-  debug_set_output (NULL, NULL);
+  debug_set_output (NULL);
 
   if (frozen_file_to_write)
     produce_frozen_state (frozen_file_to_write);
@@ -709,11 +590,6 @@ main (int argc, char *const *argv, char *const *envp)
       undivert_all ();
     }
   output_exit ();
-  free_regex ();
-  quotearg_free ();
-#ifdef DEBUG_REGEX
-  if (trace_file)
-    fclose (trace_file);
-#endif /* DEBUG_REGEX */
+  free_macro_sequence ();
   exit (retcode);
 }
