@@ -1,22 +1,22 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2004, 2005, 2006 Free
-   Software Foundation, Inc.
+   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2004, 2005, 2006,
+   2007, 2008 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This file is part of GNU M4.
+
+   GNU M4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GNU M4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301  USA
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /* We use <config.h> instead of "config.h" so that a compilation
@@ -25,6 +25,29 @@
 
 #include <config.h>
 
+#include <ctype.h>
+#include <errno.h>
+#include <stdbool.h>
+#include <stdint.h>
+#include <string.h>
+#include <sys/types.h>
+
+#include "binary-io.h"
+#include "clean-temp.h"
+#include "cloexec.h"
+#include "close-stream.h"
+#include "closein.h"
+#include "error.h"
+#include "exitfail.h"
+#include "obstack.h"
+#include "stdio--.h"
+#include "stdlib--.h"
+#include "unistd--.h"
+#include "verror.h"
+#include "xalloc.h"
+#include "xprintf.h"
+#include "xvasprintf.h"
+
 /* Canonicalize UNIX recognition macros.  */
 #if defined unix || defined __unix || defined __unix__ \
   || defined _POSIX_VERSION || defined _POSIX2_VERSION \
@@ -43,36 +66,6 @@
 # define OS2 1
 #endif
 
-#include <ctype.h>
-#include <errno.h>
-#include <string.h>
-#include <sys/types.h>
-
-#include "binary-io.h"
-#include "cloexec.h"
-#include "close-stream.h"
-#include "error.h"
-#include "exit.h"
-#include "obstack.h"
-#include "stdio--.h"
-#include "stdlib--.h"
-#include "unistd--.h"
-#include "verror.h"
-#include "xalloc.h"
-
-/* If FALSE is defined, we presume TRUE is defined too.  In this case,
-   merely typedef boolean as being int.  Or else, define these all.  */
-#ifndef FALSE
-/* Do not use `enum boolean': this tag is used in SVR4 <sys/types.h>.  */
-typedef enum { FALSE = 0, TRUE = 1 } boolean;
-#else
-typedef int boolean;
-#endif
-
-#if ! HAVE_MKSTEMP
-int mkstemp (char *);
-#endif
-
 /* Used for version mismatch, when -R detects a frozen file it can't parse.  */
 #define EXIT_MISMATCH 63
 
@@ -93,6 +86,15 @@ typedef struct string STRING;
 typedef struct token_data token_data;
 typedef void builtin_func (struct obstack *, int, token_data **);
 
+/* Gnulib's stdbool doesn't work with bool bitfields.  For nicer
+   debugging, use bool when we know it works, but use the more
+   portable unsigned int elsewhere.  */
+#if __GNUC__ > 2
+typedef bool bool_bitfield;
+#else
+typedef unsigned int bool_bitfield;
+#endif /* ! __GNUC__ */
+
 /* Take advantage of GNU C compiler source level optimization hints,
    using portable macros.  */
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 6)
@@ -127,14 +129,14 @@ extern const char *program_name;
 
 void m4_error (int, int, const char *, ...) M4_GNUC_PRINTF(3, 4);
 void m4_error_at_line (int, int, const char *, int,
-                       const char *, ...) M4_GNUC_PRINTF(5, 6);
+		       const char *, ...) M4_GNUC_PRINTF(5, 6);
 
 #define M4ERROR(Arglist) (m4_error Arglist)
 #define M4ERROR_AT_LINE(Arglist) (m4_error_at_line Arglist)
 
 #ifdef USE_STACKOVF
 void setup_stackovf_trap (char *const *, char *const *,
-                          void (*handler) (void));
+			  void (*handler) (void));
 #endif
 
 /* File: debug.c  --- debugging and tracing function.  */
@@ -173,7 +175,7 @@ extern FILE *debug;
   do								\
     {								\
       if (debug != NULL)					\
-	fprintf (debug, Fmt, Arg1);				\
+	xfprintf (debug, Fmt, Arg1);				\
     }								\
   while (0)
 
@@ -181,7 +183,7 @@ extern FILE *debug;
   do								\
     {								\
       if (debug != NULL)					\
-	fprintf (debug, Fmt, Arg1, Arg2, Arg3);			\
+	xfprintf (debug, Fmt, Arg1, Arg2, Arg3);                \
     }								\
   while (0)
 
@@ -191,7 +193,7 @@ extern FILE *debug;
       if (debug != NULL)					\
 	{							\
 	  debug_message_prefix ();				\
-	  fprintf (debug, Fmt);					\
+	  xfprintf (debug, Fmt);                                \
 	  putc ('\n', debug);					\
 	}							\
     }								\
@@ -203,7 +205,7 @@ extern FILE *debug;
       if (debug != NULL)					\
 	{							\
 	  debug_message_prefix ();				\
-	  fprintf (debug, Fmt, Arg1);				\
+	  xfprintf (debug, Fmt, Arg1);				\
 	  putc ('\n', debug);					\
 	}							\
     }								\
@@ -215,7 +217,7 @@ extern FILE *debug;
       if (debug != NULL)					\
 	{							\
 	  debug_message_prefix ();				\
-	  fprintf (debug, Fmt, Arg1, Arg2);			\
+	  xfprintf (debug, Fmt, Arg1, Arg2);			\
 	  putc ('\n', debug);					\
 	}							\
     }								\
@@ -224,7 +226,7 @@ extern FILE *debug;
 void debug_init (void);
 int debug_decode (const char *);
 void debug_flush_files (void);
-boolean debug_set_output (const char *);
+bool debug_set_output (const char *);
 void debug_message_prefix (void);
 
 void trace_prepre (const char *, int);
@@ -284,16 +286,16 @@ typedef enum token_data_type token_data_type;
 
 void input_init (void);
 token_type peek_token (void);
-token_type next_token (token_data *);
+token_type next_token (token_data *, int *);
 void skip_line (void);
 
 /* push back input */
-void push_file (FILE *, const char *);
+void push_file (FILE *, const char *, bool);
 void push_macro (builtin_func *);
 struct obstack *push_string_init (void);
 const char *push_string_finish (void);
 void push_wrapup (const char *);
-boolean pop_wrapup (void);
+bool pop_wrapup (void);
 
 /* current input file, and line */
 extern const char *current_file;
@@ -319,7 +321,9 @@ extern int current_diversion;
 extern int output_current_line;
 
 void output_init (void);
-void shipout_text (struct obstack *, const char *, int);
+void output_exit (void);
+void output_text (const char *, int);
+void shipout_text (struct obstack *, const char *, int, int);
 void make_diversion (int);
 void insert_diversion (int);
 void insert_file (FILE *);
@@ -341,11 +345,11 @@ enum symbol_lookup
 struct symbol
 {
   struct symbol *next;
-  boolean traced : 1;
-  boolean shadowed : 1;
-  boolean macro_args : 1;
-  boolean blind_no_args : 1;
-  boolean deleted : 1;
+  bool_bitfield traced : 1;
+  bool_bitfield shadowed : 1;
+  bool_bitfield macro_args : 1;
+  bool_bitfield blind_no_args : 1;
+  bool_bitfield deleted : 1;
   int pending_expansions;
 
   char *name;
@@ -366,7 +370,7 @@ struct symbol
 
 typedef enum symbol_lookup symbol_lookup;
 typedef struct symbol symbol;
-typedef void hack_symbol ();
+typedef void hack_symbol (symbol *, void *);
 
 #define HASHMAX 509		/* default, overridden by -Hsize */
 
@@ -375,7 +379,7 @@ extern symbol **symtab;
 void free_symbol (symbol *sym);
 void symtab_init (void);
 symbol *lookup_symbol (const char *, symbol_lookup);
-void hack_all_symbols (hack_symbol *, const char *);
+void hack_all_symbols (hack_symbol *, void *);
 
 /* File: macro.c  --- macro expansion.  */
 
@@ -387,9 +391,9 @@ void call_macro (symbol *, int, token_data **, struct obstack *);
 struct builtin
 {
   const char *name;
-  boolean gnu_extension : 1;
-  boolean groks_macro_args : 1;
-  boolean blind_if_no_args : 1;
+  bool_bitfield gnu_extension : 1;
+  bool_bitfield groks_macro_args : 1;
+  bool_bitfield blind_if_no_args : 1;
   builtin_func *func;
 };
 
@@ -402,13 +406,24 @@ struct predefined
 
 typedef struct builtin builtin;
 typedef struct predefined predefined;
+struct re_pattern_buffer;
+struct re_registers;
+
+/* The default sequence detects multi-digit parameters (obsolete after
+   1.4.x), and any use of extended arguments with the default ${}
+   syntax (new in 2.0).  */
+#define DEFAULT_MACRO_SEQUENCE "\\$\\({[^}]*}\\|[0-9][0-9]+\\)"
 
 void builtin_init (void);
 void define_builtin (const char *, const builtin *, symbol_lookup);
+void set_macro_sequence (const char *);
+void free_macro_sequence (void);
 void define_user_macro (const char *, const char *, symbol_lookup);
 void undivert_all (void);
 void expand_user_macro (struct obstack *, symbol *, int, token_data **);
 void m4_placeholder (struct obstack *, int, token_data **);
+void init_pattern_buffer (struct re_pattern_buffer *, struct re_registers *);
+const char *ntoa (int32_t, int);
 
 const builtin *find_builtin_by_addr (builtin_func *);
 const builtin *find_builtin_by_name (const char *);
@@ -418,15 +433,11 @@ const builtin *find_builtin_by_name (const char *);
 void include_init (void);
 void include_env_init (void);
 void add_include_directory (const char *);
-FILE *path_search (const char *, const char **);
+FILE *m4_path_search (const char *, char **);
 
 /* File: eval.c  --- expression evaluation.  */
 
-/* eval_t and unsigned_eval_t should be at least 32 bits.  */
-typedef int eval_t;
-typedef unsigned int unsigned_eval_t;
-
-boolean evaluate (const char *, eval_t *);
+bool evaluate (const char *, int32_t *);
 
 /* File: format.c  --- printf like formatting.  */
 
@@ -458,4 +469,8 @@ void reload_frozen_state (const char *);
 /* Convert a possibly-signed character to an unsigned character.  This is
    a bit safer than casting to unsigned char, since it catches some type
    errors that the cast doesn't.  */
+#if HAVE_INLINE
 static inline unsigned char to_uchar (char ch) { return ch; }
+#else
+# define to_uchar(C) ((unsigned char) (C))
+#endif
