@@ -1,22 +1,22 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2004, 2005, 2006 Free
-   Software Foundation, Inc.
+   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2004, 2005, 2006,
+   2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This file is part of GNU M4.
+
+   GNU M4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GNU M4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301  USA
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "m4.h"
@@ -131,7 +131,8 @@ threshold_diversion_CB (const void *elt, const void *threshold)
 void
 output_init (void)
 {
-  diversion_table = gl_oset_create_empty (GL_AVLTREE_OSET, cmp_diversion_CB);
+  diversion_table = gl_oset_create_empty (GL_AVLTREE_OSET, cmp_diversion_CB,
+					  NULL);
   div0.u.file = stdout;
   output_diversion = &div0;
   output_file = stdout;
@@ -214,7 +215,6 @@ m4_tmpfile (int divnum)
 
   if (output_temp_dir == NULL)
     {
-      errno = 0;
       output_temp_dir = create_temp_dir ("m4-", NULL, true);
       if (output_temp_dir == NULL)
 	M4ERROR ((EXIT_FAILURE, errno,
@@ -223,7 +223,6 @@ m4_tmpfile (int divnum)
     }
   name = m4_tmpname (divnum);
   register_temp_file (output_temp_dir, name);
-  errno = 0;
   file = fopen_temp (name, O_BINARY ? "wb+" : "w+");
   if (file == NULL)
     {
@@ -246,7 +245,6 @@ m4_tmpopen (int divnum)
   const char *name = m4_tmpname (divnum);
   FILE *file;
 
-  errno = 0;
   file = fopen_temp (name, O_BINARY ? "ab+" : "a+");
   if (file == NULL)
     M4ERROR ((EXIT_FAILURE, errno,
@@ -254,6 +252,11 @@ m4_tmpopen (int divnum)
   else if (set_cloexec_flag (fileno (file), true) != 0)
     M4ERROR ((warning_status, errno,
 	      "Warning: cannot protect diversion across forks"));
+  /* POSIX states that it is undefined whether an append stream starts
+     at offset 0 or at the end.  We want the beginning.  */
+  else if (fseeko (file, 0, SEEK_SET) != 0)
+    M4ERROR ((EXIT_FAILURE, errno,
+	      "cannot seek to beginning of diversion"));
   return file;
 }
 
@@ -372,7 +375,7 @@ make_room_for (int length)
       if (selected_diversion)
 	{
 	  FILE *file = selected_diversion->u.file;
-	  selected_diversion->u.file = 0;
+	  selected_diversion->u.file = NULL;
 	  if (m4_tmpclose (file) != 0)
 	    M4ERROR ((0, errno, "cannot close temporary file for diversion"));
 	}
@@ -421,11 +424,14 @@ output_character_helper (int character)
 | to a diversion file or an in-memory diversion buffer.			  |
 `------------------------------------------------------------------------*/
 
-static void
+void
 output_text (const char *text, int length)
 {
   int count;
 
+  if (!output_diversion || !length)
+    return;
+
   if (!output_file && length > output_unused)
     make_room_for (length);
 
@@ -443,23 +449,25 @@ output_text (const char *text, int length)
     }
 }
 
-/*-------------------------------------------------------------------------.
-| Add some text into an obstack OBS, taken from TEXT, having LENGTH	   |
-| characters.  If OBS is NULL, rather output the text to an external file  |
-| or an in-memory diversion buffer.  If OBS is NULL, and there is no	   |
-| output file, the text is discarded.					   |
-|									   |
-| If we are generating sync lines, the output have to be examined, because |
-| we need to know how much output each input line generates.  In general,  |
-| sync lines are output whenever a single input lines generates several	   |
-| output lines, or when several input lines does not generate any output.  |
-`-------------------------------------------------------------------------*/
+/*--------------------------------------------------------------------.
+| Add some text into an obstack OBS, taken from TEXT, having LENGTH   |
+| characters.  If OBS is NULL, output the text to an external file    |
+| or an in-memory diversion buffer instead.  If OBS is NULL, and      |
+| there is no output file, the text is discarded.  LINE is the line   |
+| where the token starts (not necessarily current_line, in the case   |
+| of multiline tokens).                                               |
+|                                                                     |
+| If we are generating sync lines, the output has to be examined,     |
+| because we need to know how much output each input line generates.  |
+| In general, sync lines are output whenever a single input lines     |
+| generates several output lines, or when several input lines do not  |
+| generate any output.                                                |
+`--------------------------------------------------------------------*/
 
 void
-shipout_text (struct obstack *obs, const char *text, int length)
+shipout_text (struct obstack *obs, const char *text, int length, int line)
 {
   static bool start_of_output_line = true;
-  char line[20];
   const char *cursor;
 
   /* If output goes to an obstack, merely add TEXT to it.  */
@@ -500,43 +508,64 @@ shipout_text (struct obstack *obs, const char *text, int length)
 	output_text (text, length);
       }
   else
-    for (; length-- > 0; text++)
-      {
-	if (start_of_output_line)
-	  {
-	    start_of_output_line = false;
-	    output_current_line++;
-
+    {
+      /* Check for syncline only at the start of a token.  Multiline
+	 tokens, and tokens that are out of sync but in the middle of
+	 the line, must wait until the next raw newline triggers a
+	 syncline.  */
+      if (start_of_output_line)
+	{
+	  start_of_output_line = false;
+	  output_current_line++;
 #ifdef DEBUG_OUTPUT
-	    printf ("DEBUG: cur %d, cur out %d\n",
-		    current_line, output_current_line);
+	  xfprintf (stderr, "DEBUG: line %d, cur %d, cur out %d\n",
+		   line, current_line, output_current_line);
 #endif
 
-	    /* Output a `#line NUM' synchronization directive if needed.
-	       If output_current_line was previously given a negative
-	       value (invalidated), rather output `#line NUM "FILE"'.  */
+	  /* Output a `#line NUM' synchronization directive if needed.
+	     If output_current_line was previously given a negative
+	     value (invalidated), output `#line NUM "FILE"' instead.  */
 
-	    if (output_current_line != current_line)
-	      {
-		sprintf (line, "#line %d", current_line);
-		for (cursor = line; *cursor; cursor++)
-		  OUTPUT_CHARACTER (*cursor);
-		if (output_current_line < 1 && current_file[0] != '\0')
-		  {
-		    OUTPUT_CHARACTER (' ');
-		    OUTPUT_CHARACTER ('"');
-		    for (cursor = current_file; *cursor; cursor++)
-		      OUTPUT_CHARACTER (*cursor);
-		    OUTPUT_CHARACTER ('"');
-		  }
-		OUTPUT_CHARACTER ('\n');
-		output_current_line = current_line;
-	      }
-	  }
-	OUTPUT_CHARACTER (*text);
-	if (*text == '\n')
-	  start_of_output_line = true;
-      }
+	  if (output_current_line != line)
+	    {
+	      OUTPUT_CHARACTER ('#');
+	      OUTPUT_CHARACTER ('l');
+	      OUTPUT_CHARACTER ('i');
+	      OUTPUT_CHARACTER ('n');
+	      OUTPUT_CHARACTER ('e');
+	      OUTPUT_CHARACTER (' ');
+	      for (cursor = ntoa (line, 10); *cursor; cursor++)
+		OUTPUT_CHARACTER (*cursor);
+	      if (output_current_line < 1 && current_file[0] != '\0')
+		{
+		  OUTPUT_CHARACTER (' ');
+		  OUTPUT_CHARACTER ('"');
+		  for (cursor = current_file; *cursor; cursor++)
+		    OUTPUT_CHARACTER (*cursor);
+		  OUTPUT_CHARACTER ('"');
+		}
+	      OUTPUT_CHARACTER ('\n');
+	      output_current_line = line;
+	    }
+	}
+
+      /* Output the token, and track embedded newlines.  */
+      for (; length-- > 0; text++)
+	{
+	  if (start_of_output_line)
+	    {
+	      start_of_output_line = false;
+	      output_current_line++;
+#ifdef DEBUG_OUTPUT
+	      xfprintf (stderr, "DEBUG: line %d, cur %d, cur out %d\n",
+		       line, current_line, output_current_line);
+#endif
+	    }
+	  OUTPUT_CHARACTER (*text);
+	  if (*text == '\n')
+	    start_of_output_line = true;
+	}
+    }
 }
 
 /* Functions for use by diversions.  */
@@ -682,13 +711,9 @@ insert_diversion_helper (m4_diversion *diversion)
 	output_text (diversion->u.buffer, diversion->used);
       else
 	{
-	  if (!diversion->u.file && diversion->used)
+	  if (!diversion->u.file)
 	    diversion->u.file = m4_tmpopen (diversion->divnum);
-	  if (diversion->u.file)
-	    {
-	      rewind (diversion->u.file);
-	      insert_file (diversion->u.file);
-	    }
+	  insert_file (diversion->u.file);
 	}
 
       output_current_line = -1;
@@ -783,21 +808,21 @@ freeze_diversions (FILE *file)
   while (gl_oset_iterator_next (&iter, &elt))
     {
       m4_diversion *diversion = (m4_diversion *) elt;;
-      if (diversion->size || diversion->u.file)
+      if (diversion->size || diversion->used)
 	{
 	  if (diversion->size)
-	    fprintf (file, "D%d,%d\n", diversion->divnum, diversion->used);
+	    xfprintf (file, "D%d,%d\n", diversion->divnum, diversion->used);
 	  else
 	    {
 	      struct stat file_stat;
-	      fflush (diversion->u.file);
+	      diversion->u.file = m4_tmpopen (diversion->divnum);
 	      if (fstat (fileno (diversion->u.file), &file_stat) < 0)
 		M4ERROR ((EXIT_FAILURE, errno, "cannot stat diversion"));
 	      if (file_stat.st_size < 0
 		  || file_stat.st_size != (unsigned long int) file_stat.st_size)
 		M4ERROR ((EXIT_FAILURE, 0, "diversion too large"));
-	      fprintf (file, "D%d,%lu", diversion->divnum,
-		       (unsigned long int) file_stat.st_size);
+	      xfprintf (file, "D%d,%lu\n", diversion->divnum,
+			(unsigned long int) file_stat.st_size);
 	    }
 
 	  insert_diversion_helper (diversion);
@@ -811,5 +836,5 @@ freeze_diversions (FILE *file)
   /* Save the active diversion number, if not already.  */
 
   if (saved_number != last_inserted)
-    fprintf (file, "D%d,0\n\n", saved_number);
+    xfprintf (file, "D%d,0\n\n", saved_number);
 }
