@@ -133,7 +133,8 @@ m4_path_search (const char *file, char **result)
   if (fp != NULL)
     {
       if (set_cloexec_flag (fileno (fp), true) != 0)
-	m4_warn (errno, NULL, _("cannot protect input file across forks"));
+	M4ERROR ((warning_status, errno,
+		  "Warning: cannot protect input file across forks"));
       if (result)
 	*result = xstrdup (file);
       return fp;
@@ -162,7 +163,8 @@ m4_path_search (const char *file, char **result)
 	  if (debug_level & DEBUG_TRACE_PATH)
 	    DEBUG_MESSAGE2 ("path search for `%s' found `%s'", file, name);
 	  if (set_cloexec_flag (fileno (fp), true) != 0)
-	    m4_warn (errno, NULL, _("cannot protect input file across forks"));
+	    M4ERROR ((warning_status, errno,
+		      "Warning: cannot protect input file across forks"));
 	  if (result)
 	    *result = name;
 	  else
