@@ -1,7 +1,7 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2003, 2006, 2007, 2008
-   Free Software Foundation, Inc.
+   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2003, 2006, 2007 Free
+   Software Foundation, Inc.
 
    This file is part of GNU M4.
 
@@ -32,6 +32,7 @@
    will then always be the first found.  */
 
 #include "m4.h"
+#include <limits.h>
 
 #ifdef DEBUG_SYM
 /* When evaluating hash table performance, this profiling code shows
@@ -106,7 +107,12 @@ symtab_init (void)
     s[i] = NULL;
 
 #ifdef DEBUG_SYM
-  atexit (show_profile); /* Ignore failure, since this is debug code.  */
+  {
+    int e = atexit(show_profile);
+    if (e != 0)
+      M4ERROR ((warning_status, 0,
+		"INTERNAL ERROR: unable to show symtab profile"));
+  }
 #endif /* DEBUG_SYM */
 }
 
@@ -301,7 +307,8 @@ lookup_symbol (const char *name, symbol_lookup mode)
       return NULL;
 
     default:
-      assert (!"symbol_lookup");
+      M4ERROR ((warning_status, 0,
+		"INTERNAL ERROR: invalid mode to symbol_lookup ()"));
       abort ();
     }
 }
@@ -350,7 +357,7 @@ symtab_debug (void)
   int delete;
   static int i;
 
-  while (next_token (&td, NULL, NULL, false, "<debug>") == TOKEN_WORD)
+  while (next_token (&td) == TOKEN_WORD)
     {
       text = TOKEN_DATA_TEXT (&td);
       if (*text == '_')
